/****************************************************************************
       Module: FLASH.C
  Description: This source code is based on example code written by Holger
               Dienst of Infineon
Version     Date           Initials    Description
1.0.0       26.02.1999     HD          Initial
1.0.1       06.06.2000     MH          Added code to automatically check size
                                       of external SRAM (previously assumed
                                       size was 1MB)
****************************************************************************/
#include <stdio.h>
#include <string.h>

unsigned int Read_ID( unsigned int uiFlashBase );
unsigned int Chip_Erase(unsigned int uiID, unsigned int uiFlashBase);
unsigned int Program_Word(unsigned int uiID, unsigned int uiAddress, unsigned int uiValue, unsigned int uiFlashBase );
unsigned int Intel_Flash_ID( unsigned int uiFlashBase );
unsigned int Intel_Chip_Erase(unsigned int uiID, unsigned int uiFlashBase);
unsigned int Intel_Program_Word( unsigned int uiAddress, unsigned int uiValue );
unsigned int AMD_Flash_ID( unsigned int uiFlashBase );
unsigned int AMD_Chip_Erase( unsigned int uiFlashBase );
unsigned int AMD_Program_Word( unsigned int uiAddress, unsigned int uiValue, unsigned int uiFlashBase );

/* Config address */
#define   SETUP_ADDR      0x1554
#define UNLOCK_ADDR      0xAA8

/* AMD Flash commands */
#define   AMDSETUP        0x00AA00AA
#define   AMDUNLOCK        0x00550055
#define   AMDREAD_ID       0x00900090
#define   AMDPROGRAM       0x00A000A0
#define   AMDCHIP_ERASE1   0x00800080
#define   AMDCHIP_ERASE2   0x00100010
#define   AMDSECTOR_ERASE   0x00300030
#define   AMDRESET          0x00F000F0

/* INTEL Flash commands */
#define   INTELREAD_ID      0x00900090
#define   INTELREAD_ARRAY   0x00FF00FF
#define   INTELREAD_STAT    0x00700070
#define   INTELCLR_STAT    0x00500050
#define   INTELBLK_ERASE    0x00200020
#define   INTELRESUME       0x00D000D0
#define   INTELSUSPEND      0x00B000B0
#define   INTELPROG_STD   0x00400040
#define   INTELPROG_ALT   0x00100010
#define   INTELSET_CFG1   0x00600060
#define   INTELSET_CFG2   0x00030003

/* bits for Polling (end of operation) */
#define DQ7            0x00800080
#define DQ5            0x00200020

#define   STAT_WSMS       0x00800080
#define   STAT_ESS        0x00400040
#define   STAT_ES         0x00200020
#define   STAT_PS         0x00100010
#define   STAT_VPPS       0x00080008
#define   STAT_PSS        0x00040004
#define   STAT_DPS        0x00020002
#define   STAT_R          0x00010001

/* result of command */
#define SUCCESS         0x0
#define FAILURE         0x1
#define WRITE_ERROR      0x3
#define DATA_ERROR      0x4

#define MANU_ID_AMD       0x0001
#define MANU_ID_TOSHIBA   0x0098
#define MANU_ID_INTEL     0x0089

#define   AM29LV200TB_ID     0x223B
#define   AM29LV200BB_ID     0x22BF
#define   AM29LV400TB_ID     0x22B9
#define   AM29LV400BB_ID     0x22BA
#define   AM29LV800TB_ID     0x22DA
#define   AM29LV800BB_ID     0x225B
#define   AM29LV160TB_ID     0x22C4
#define   AM29LV160BB_ID     0x2249
#define   AM29BL162C_ID     0x2203
#define   AM29BL802C_ID     0x2281

#define   DT28F800F3T_ID     0x88F1
#define   DT28F800F3B_ID     0x88F2
#define   DT28F160F3T_ID     0x88F3
#define   DT28F160F3B_ID     0x88F4

#define   TC58FVT400FT_ID     0x00CD
#define   TC58FVB400FT_ID     0x004C
#define   TC58FVT800FT_ID     0x004F
#define   TC58FVB800FT_ID     0x00CE
#define   TC58FVT160FT_ID     0x00C2
#define   TC58FVB160FT_ID     0x0043

#define   KB_1_SIZE           0x00000400
#define   KB_2_SIZE           0x00000800
#define   KB_4_SIZE           0x00001000
#define   KB_8_SIZE           0x00002000
#define   KB_16_SIZE          0x00004000
#define   KB_32_SIZE          0x00008000
#define   KB_64_SIZE          0x00010000
#define   KB_128_SIZE         0x00020000
#define   KB_256_SIZE         0x00040000
#define   KB_512_SIZE         0x00080000
#define   MB_1_SIZE           0x00100000
#define   MB_2_SIZE           0x00200000
#define   MB_4_SIZE           0x00400000
#define   MB_8_SIZE           0x00800000
#define   MB_16_SIZE          0x01000000
#define   MB_32_SIZE          0x02000000
#define   MB_64_SIZE          0x04000000
#define   MB_128_SIZE         0x08000000

// Global string for containing status messages...
char szStatusMessage[7][80];

void main (void)
{     
   char szFlashName[20];

   unsigned int uiBaseFlashAddr;
   unsigned int uiCurrentFlashAddr;
   unsigned int uiFlashSize;

   unsigned int uiBaseSramAddr;
   unsigned int uiCurrentSramAddr;
   unsigned int uiSramSize;

   unsigned int uiCurrentCount;
   unsigned int uiID;
   unsigned int i;

   // Initialise the status strings...
   for (i=0; i < 7; i++)
      szStatusMessage[i][0] = '\0';

   strcpy(szStatusMessage[0], "FLASH Programmer,  Version 1.0.1,  6-Jun-2000");

   // Setup base address of flash...
   uiBaseFlashAddr = *(unsigned int *)0xF0000520;
   uiBaseFlashAddr &= 0xFFFFF000;

   // Setup base address of SRAM...
   uiBaseSramAddr = *(unsigned int *)0xF0000524;
   uiBaseSramAddr &= 0xFFFFF000;

   // Setup size of SRAM...
   uiSramSize = *(unsigned int *)0xF0000524;
   uiSramSize >>= 4;
   uiSramSize &= 0x0000000F;
   uiSramSize = KB_4_SIZE << (15 - uiSramSize);

   uiID = Read_ID( uiBaseFlashAddr );

   if (((uiID>>16) != MANU_ID_AMD) &&
       ((uiID>>16) != MANU_ID_TOSHIBA) &&
       ((uiID>>16) != MANU_ID_INTEL))
      {
      sprintf(szStatusMessage[1], "Unknown flash manufacturer ID [%X]", uiID>>16);
      goto FlashErrorHandler;
      }

   switch (uiID & 0xFFFF)
      {
      case AM29LV200TB_ID:
         strcpy(szFlashName, "AM29LV200TB");
         uiFlashSize = KB_512_SIZE;
         break;
      case AM29LV200BB_ID:
         strcpy(szFlashName, "AM29LV200BB");
         uiFlashSize = KB_512_SIZE;
         break;
      case TC58FVT400FT_ID:
         strcpy(szFlashName, "TC58FVT400FT");
         uiFlashSize = MB_1_SIZE;
         break;
      case TC58FVB400FT_ID:
         strcpy(szFlashName, "TC58FVB400FT");
         uiFlashSize = MB_1_SIZE;
         break;
      case AM29LV400TB_ID:
         strcpy(szFlashName, "AM29LV400TB");
         uiFlashSize = MB_1_SIZE;
         break;
      case AM29LV400BB_ID:
         strcpy(szFlashName, "AM29LV400BB");
         uiFlashSize = MB_1_SIZE;
         break;
      case TC58FVT800FT_ID:
         strcpy(szFlashName, "TC58FVT800FT");
         uiFlashSize = MB_2_SIZE;
         break;
      case TC58FVB800FT_ID:
         strcpy(szFlashName, "TC58FVB800FT");
         uiFlashSize = MB_2_SIZE;
         break;
      case AM29LV800TB_ID:
         strcpy(szFlashName, "AM29LV800TB");
         uiFlashSize = MB_2_SIZE;
         break;
      case AM29LV800BB_ID: 
         strcpy(szFlashName, "AM29LV800BB");
         uiFlashSize = MB_2_SIZE;
         break;
      case AM29BL802C_ID:
         strcpy(szFlashName, "AM29BL802C");
         uiFlashSize = MB_2_SIZE/4;
         break;
      case DT28F800F3T_ID:
         strcpy(szFlashName, "DT28F800F3T");
         uiFlashSize = MB_2_SIZE;
         break;
      case DT28F800F3B_ID:
         strcpy(szFlashName, "DT28F800F3B");
         uiFlashSize = MB_2_SIZE;
         break;
      case TC58FVT160FT_ID:
         strcpy(szFlashName, "TC58FVT160FT");
         uiFlashSize = MB_4_SIZE;
         break;
      case TC58FVB160FT_ID:
         strcpy(szFlashName, "TC58FVB160FT");
         uiFlashSize = MB_4_SIZE;
         break;
      case AM29LV160TB_ID:
         strcpy(szFlashName, "AM29LV160TB");
         uiFlashSize = MB_4_SIZE/4;
         break;
      case AM29LV160BB_ID: 
         strcpy(szFlashName, "AM29LV160BB");
         uiFlashSize = MB_4_SIZE;
         break;
      case AM29BL162C_ID:
         strcpy(szFlashName, "AM29BL162C");
         uiFlashSize = MB_4_SIZE;
         break;
      case DT28F160F3B_ID:
         strcpy(szFlashName, "DT28F160F3B");
         uiFlashSize = MB_4_SIZE;
         break;
      case DT28F160F3T_ID:
         strcpy(szFlashName, "DT28F160F3T");
         uiFlashSize = MB_4_SIZE;
         break;
      default:
         strcpy(szStatusMessage[1], "Unknown / Unsupported flash size");
         goto FlashErrorHandler;
      }

   if (uiSramSize >= MB_1_SIZE)
      sprintf(szStatusMessage[1], "Determined SRAM is of %dMB size", uiSramSize/MB_1_SIZE);
   else 
      sprintf(szStatusMessage[1], "Determined SRAM is of %dKB size",  uiSramSize/KB_1_SIZE);

   if (uiFlashSize >= MB_1_SIZE)
      sprintf(szStatusMessage[2], "Determined flash is of %dMB size, type is %s", uiFlashSize/MB_1_SIZE, szFlashName);
   else 
      sprintf(szStatusMessage[2], "Determined flash is of %dKB size, type is %s", uiFlashSize/KB_1_SIZE, szFlashName);

   /*
      We can only program up to the size of the external SRAM
   */
   if (uiFlashSize > uiSramSize)
      uiFlashSize = uiSramSize;

   strcpy(szStatusMessage[3], "Erasing flash device...");
   Chip_Erase( uiID, uiBaseFlashAddr );
   strcat(szStatusMessage[3], "completed");
   
   strcpy(szStatusMessage[4], "Programming flash device...");

   uiCurrentCount       = uiFlashSize / 4;
   uiCurrentSramAddr    = uiBaseSramAddr;
   uiCurrentFlashAddr   = uiBaseFlashAddr;

   while (uiCurrentCount--)
      {   
      if (Program_Word(uiID,
                       uiCurrentFlashAddr,
                       *(unsigned int *)uiCurrentSramAddr,
                       uiBaseFlashAddr) != SUCCESS)
         {
         // Failed to program.....try again...
         uiCurrentCount++;
         continue;
         }

      uiCurrentFlashAddr += 4;
      uiCurrentSramAddr  += 4;
      }

   strcat(szStatusMessage[4], "completed");

   strcpy(szStatusMessage[5], "Verifying flash device...");

   uiCurrentCount       = uiFlashSize / 4;
   uiCurrentSramAddr    = uiBaseSramAddr;
   uiCurrentFlashAddr   = uiBaseFlashAddr;

   while (uiCurrentCount--)
      {   
      if (*(unsigned int *)uiCurrentSramAddr != *(unsigned int *)uiCurrentFlashAddr)
         {
         sprintf(szStatusMessage[6], "Failed to verify at address %8.8X", uiCurrentSramAddr);
         goto FlashErrorHandler;
         }
      
      uiCurrentFlashAddr += 4;
      uiCurrentSramAddr  += 4;
      }
   strcat(szStatusMessage[5], "completed");

   strcpy(szStatusMessage[6], "Flash programming completed successfully.");


   // Flow through and loop for ever.

FlashErrorHandler:
   _debug();
   while (1);
}

/* read id */
unsigned int Read_ID( unsigned int uiFlashBase )
{
   unsigned int uiID;

   uiID = AMD_Flash_ID( uiFlashBase );

   if (((uiID>>16) != MANU_ID_AMD) &&
       ((uiID>>16) != MANU_ID_TOSHIBA))
      uiID = Intel_Flash_ID(uiFlashBase);

   return uiID;
}

unsigned int Chip_Erase(unsigned int uiID, unsigned int uiFlashBase)
{
   unsigned int uiRetVal;

   if ((uiID>>16) == MANU_ID_INTEL)
      uiRetVal = Intel_Chip_Erase(uiID, uiFlashBase);
   else
      uiRetVal = AMD_Chip_Erase(uiFlashBase);

   return uiRetVal;
}

unsigned int Program_Word(unsigned int uiID, unsigned int uiAddress, unsigned int uiValue, unsigned int uiFlashBase )
{
   unsigned int uiRetVal;

   if ((uiID>>16) == MANU_ID_INTEL)
      uiRetVal = Intel_Program_Word(uiAddress, uiValue);
   else
      uiRetVal = AMD_Program_Word(uiAddress, uiValue, uiFlashBase);

   return uiRetVal;
}

unsigned int AMD_Flash_ID( unsigned int uiFlashBase )
{
   register unsigned int *puiAddress;
   register unsigned int uiData = 0x0;
   register unsigned int uiHelp = 0x0;
   /* first bus cycle */
   puiAddress = (unsigned int *)(SETUP_ADDR+uiFlashBase);
   *puiAddress = AMDSETUP;
   /* second bus cycle */
   puiAddress = (unsigned int *)(UNLOCK_ADDR+uiFlashBase);
   *puiAddress = AMDUNLOCK;
   /* third bus cycle */
   puiAddress = (unsigned int *)(SETUP_ADDR+uiFlashBase);
   *puiAddress = AMDREAD_ID;
   puiAddress = (unsigned int *)(uiFlashBase);
   uiData = *puiAddress;
   uiHelp = (uiData & 0xffff0000);
   if ((uiData & 0xffff) != (uiHelp>>16))
      return 0;
   else
      uiData = *(puiAddress+1);
   uiHelp |= (uiData & 0xffff0000)>>16;
   if ((uiData & 0xffff) != (uiHelp & 0xffff))
      return 0;

   *puiAddress = AMDRESET;

   return uiHelp;
}

/* write value at address and wait for success */
unsigned int AMD_Program_Word( unsigned int uiAddress, unsigned int uiValue, unsigned int uiFlashBase )
{
   register unsigned int uiData;
   volatile register unsigned int *puiAddress;
   /* first bus cycle */
   puiAddress = (unsigned int *)(SETUP_ADDR+uiFlashBase);
   *puiAddress = AMDSETUP ;
   /* second bus cycle */
   puiAddress = (unsigned int *)(UNLOCK_ADDR+uiFlashBase);
   *puiAddress = AMDUNLOCK;
   /* third bus cycle */
   puiAddress = (unsigned int *)(SETUP_ADDR+uiFlashBase);
   *puiAddress = AMDPROGRAM;
   /* fourth bus cycle */
   puiAddress = (unsigned int *)uiAddress;
   *puiAddress = uiValue;

   /* data polling */
   do
      {
      /* read value */
      uiData = *puiAddress;
      /* check data ready */
      if ((uiData & DQ7) == (uiValue & DQ7))
         {
         *puiAddress = AMDRESET;
         return SUCCESS;
         }
      /* check of time out */
      if ((uiData & DQ5) == DQ5)
         {
         uiData = *puiAddress;
         if ((uiData & DQ7) == (uiValue & DQ7))
            {
            *puiAddress = AMDRESET;
            return SUCCESS;
            }
         else
            {
            *puiAddress = AMDRESET;
            return FAILURE;
            }
         }
      }
   while ((uiData & DQ5) != DQ5);
}

/* chip erase */
unsigned int AMD_Chip_Erase( unsigned int uiFlashBase )
{
   register unsigned int *puiAddress;
   register unsigned int uiData = 0x0;
   /* first bus cycle */
   puiAddress = (unsigned int *)(SETUP_ADDR+uiFlashBase);
   *puiAddress = AMDSETUP;
   /* second bus cycle */
   puiAddress = (unsigned int *)(UNLOCK_ADDR+uiFlashBase);
   *puiAddress = AMDUNLOCK;
   /* third bus cycle */
   puiAddress = (unsigned int *)(SETUP_ADDR+uiFlashBase);
   *puiAddress = AMDCHIP_ERASE1;
   /* fourth bus cycle */
   puiAddress = (unsigned int *)(SETUP_ADDR+uiFlashBase);
   *puiAddress = AMDSETUP;
   /* fifth bus cycle */
   puiAddress = (unsigned int *)(UNLOCK_ADDR+uiFlashBase);
   *puiAddress = AMDUNLOCK;
   /* sixth bus cycle */
   puiAddress = (unsigned int *)(SETUP_ADDR+uiFlashBase);
   *puiAddress = AMDCHIP_ERASE2;
   /* Polling while DQ7 == LOW */
   puiAddress = (unsigned int *)uiFlashBase;
   while ((uiData & DQ7) != DQ7) 
      {
      uiData = *puiAddress;
      }
   *puiAddress = AMDRESET;

   return SUCCESS;
}

unsigned int Intel_Flash_ID( unsigned int uiFlashBase )
{
   volatile unsigned int *puiAddress;
   unsigned int uiData = 0x0;
   unsigned int uiHelp = 0x0;

   /* first bus cycle */
   puiAddress = (unsigned int *)(uiFlashBase);
   *puiAddress = INTELREAD_ID;

   uiData = *puiAddress;
   uiHelp = (uiData & 0xffff0000);
   if ((uiData & 0xffff) != (uiHelp>>16))
      return 0;
   else
      uiData = *(puiAddress+1);
   uiHelp |= (uiData & 0xffff0000)>>16;
   if ((uiData & 0xffff) != (uiHelp & 0xffff))
      return 0;

   *puiAddress = INTELREAD_ARRAY;
   return uiHelp;
}

unsigned int Intel_Chip_Erase(unsigned int uiID, unsigned int uiFlashBase)
{
   volatile unsigned int *puiAddress;
   unsigned int uiCnt, uiSectorNumbers, uiTop;
   unsigned int uiStatus;

   uiTop = 0;

   switch(uiID & 0xFFFF)
      {
      case DT28F800F3T_ID:
         uiTop = 1;
      case DT28F800F3B_ID:
         uiSectorNumbers = 23;
         break;
      case DT28F160F3T_ID:
         uiTop = 1;
      case DT28F160F3B_ID:
         uiSectorNumbers = 39;
         break;
      default:
         return 1;
      }

   puiAddress = (unsigned int *)(uiFlashBase);

   for (uiCnt=0; uiCnt<uiSectorNumbers; uiCnt++)
      {
      *puiAddress = INTELBLK_ERASE;
      *puiAddress = INTELRESUME;
      do
         {
         uiStatus = *puiAddress;
         }
      while ((uiStatus & STAT_WSMS) != STAT_WSMS);
      if (uiStatus & STAT_ES)
         return FAILURE;
      if (uiTop)
         {
         if (uiCnt<(uiSectorNumbers-8))
            puiAddress += 0x2000;
         else
            puiAddress += 0x400;
         }
      else
         {
         if (uiCnt<8)
            puiAddress += 0x400;
         else
            puiAddress += 0x2000;
         }
      }
   *puiAddress = INTELREAD_ARRAY;
   return SUCCESS;
}

unsigned int Intel_Program_Word( unsigned int uiAddress, unsigned int uiValue )
{
   volatile unsigned int *puiAddress;
   unsigned int uiStatus;

   puiAddress = (unsigned int *)(uiAddress);

   *puiAddress = INTELPROG_STD;
   *puiAddress = uiValue;
   do
      {
      uiStatus = *puiAddress;
      }
   while ((uiStatus & STAT_WSMS) != STAT_WSMS);

   if (uiStatus & STAT_PS)
      {
      *puiAddress = INTELPROG_ALT;
      *puiAddress = uiValue;
      do
         {
         uiStatus = *puiAddress;
         }
      while ((uiStatus & STAT_WSMS) != STAT_WSMS);

      if (uiStatus & STAT_PS)
         return FAILURE;
      }
   *puiAddress = INTELREAD_ARRAY;
   return SUCCESS;
}
